from torch.autograd import Function


class RevGrad(Function):
    @staticmethod
    def forward(ctx, input_):
        ctx.save_for_backward(input_)
        output = input_
        return output

    @staticmethod
    def backward(ctx, grad_output):
        grad_input = None
        if ctx.needs_input_grad[0]:
            grad_input = -grad_output
        return grad_input


revgrad = RevGrad.apply



from .functional import revgrad
from torch.nn import Module


class RevGrad(Module):
    def __init__(self, *args, **kwargs):
        """
        A gradient reversal layer.

        This layer has no parameters, and simply reverses the gradient
        in the backward pass.
        """

        super().__init__(*args, **kwargs)

    def forward(self, input_):
        return revgrad(input_)



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['pytorch_revgrad']

package_data = \
{'': ['*']}

install_requires = \
['torch>=1.0,<2.0']

setup_kwargs = {
    'name': 'pytorch-revgrad',
    'version': '0.0.1',
    'description': 'Gradient reversal layer for pytorch.',
    'long_description': '# `pytorch-revgrad`\n\nThis package implements a gradient reversal layer for pytorch modules.\n\n## Example usage\n\n```python\nimport torch\n\nfrom pytorch_revgrad import RevGrad\n\nmodel = torch.nn.Sequential(\n    torch.nn.Linear(10, 5),\n    torch.nn.Linear(5, 2),\n    RevGrad()\n)\n```\n',
    'author': 'Jan Freyberg',
    'author_email': 'jan.freyberg@gmail.com',
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
}


setup(**setup_kwargs)
